# -*- coding: utf-8 -*-
from sklearn.svm import LinearSVC
import numpy as np
import sys
import os
import heapq
import glob
import collections


if len(sys.argv) == 1:
  exit("usage : python {0} [data file]".format(sys.argv[0]))

word_dict = collections.defaultdict(lambda: len(word_dict))
label_dict = collections.defaultdict(lambda: len(label_dict))


file = sys.argv[1]
data_words = []
data_labels = []


with open(file) as f:
  for line in f:
    words = line.rstrip().split()

    label = words.pop(0)
    lid = label_dict[label]
    data_labels.append(lid)
    
    wids = set([word_dict[w] for w in words])   #uniq
    data_words.append(wids)

x_data = np.zeros((len(data_words), len(word_dict)), int)
y_data = np.array(data_labels, int)

for i,wids in enumerate(data_words):
  for wid in wids:
    x_data[i, wid] = 1



print("vectorize done")


#split
train_size = int(len(x_data) * 0.9)
x_train,x_test = x_data[:train_size],x_data[train_size:]
y_train,y_test = y_data[:train_size],y_data[train_size:]


# learning
estimator = LinearSVC(C = 1.0)
estimator.fit(x_train, y_train)
print("leaning done")


y_pred = estimator.predict(x_test)

print(y_pred)
print(y_test)

res = [p == t for (p,t) in zip(y_pred, y_test)]
a = len(res)
t = sum(res)
rate = float(t) / a
print("accuracy {0} / {1} : {2}".format(t, a, rate))



word_dict_rev = [x[0] for x in sorted(word_dict.items(), key=lambda x:x[1])]
label_dict_rev = [x[0] for x in sorted(label_dict.items(), key=lambda x:x[1])]


for lid,coef in enumerate(estimator.coef_):
  print("---------")
  print("label {0:2} {1}".format(lid, label_dict_rev[lid]))
  
  #print top 10
  w_list = heapq.nlargest(10, enumerate(coef), key=lambda x:x[1])
  for i,score in w_list:
    print(" {0:6d}: {1} ({2})".format(i, word_dict_rev[i], score))












